// Serial.cpp

#include "stdafx.h"
#include "Serial.h"

CSerial::CSerial()
{
	memset(&m_OverlappedRead, 0, sizeof(OVERLAPPED));
 	memset(&m_OverlappedWrite, 0, sizeof(OVERLAPPED));
	m_hIDComDev = NULL;
	m_bOpened = false;
	m_nPort = -1;
}

CSerial::~CSerial()
{
	Close();
}

bool CSerial::Open(int nPort, int nBaud)
{
	CString strPort;
	strPort.Format(_T("\\\\.\\COM%d"), nPort);
	m_nPort = nPort;

	return Open(strPort, nBaud);
}

bool CSerial::Open(LPCTSTR szPort, int nBaud)
{
	if(m_bOpened) 
		return true;

	const TCHAR *p = _tcsstr(szPort, _T("COM"));
	if(p)
		m_nPort = _tstoi(p+3);
	else
		m_nPort = -1;

	DCB dcb;

	m_hIDComDev = CreateFile(szPort, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED, NULL);

	if(m_hIDComDev == NULL) 
		return false;

	memset(&m_OverlappedRead, 0, sizeof(OVERLAPPED));
 	memset(&m_OverlappedWrite, 0, sizeof(OVERLAPPED));

	COMMTIMEOUTS CommTimeOuts;
	CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF;
	CommTimeOuts.ReadTotalTimeoutMultiplier = 0;
	CommTimeOuts.ReadTotalTimeoutConstant = 0;
	CommTimeOuts.WriteTotalTimeoutMultiplier = 0;
	CommTimeOuts.WriteTotalTimeoutConstant = 5000;
	SetCommTimeouts(m_hIDComDev, &CommTimeOuts);

	m_OverlappedRead.hEvent = CreateEvent(NULL, true, false, NULL);
	m_OverlappedWrite.hEvent = CreateEvent(NULL, true, false, NULL);

	dcb.DCBlength = sizeof(DCB);
	GetCommState(m_hIDComDev, &dcb);

//	dcb.BaudRate = nBaud;
//	dcb.ByteSize = 8;

	dcb.BaudRate	= nBaud;
	dcb.ByteSize	= 8;
    dcb.StopBits    = ONESTOPBIT;
    dcb.Parity      = NOPARITY;
	dcb.fDtrControl = DTR_CONTROL_DISABLE;
	dcb.fOutX		= false;
	dcb.fInX		= false;
	dcb.fRtsControl = RTS_CONTROL_ENABLE; // disabling RTS prevented the 5235/5300 from communicating. May 
	dcb.fOutxCtsFlow = false;
	dcb.fOutxDsrFlow = false;
//	dcb.EofChar		= 10;


	if(    !SetCommState(m_hIDComDev, &dcb)
		|| !SetupComm(m_hIDComDev, 10000, 10000)
		|| m_OverlappedRead.hEvent == NULL
		|| m_OverlappedWrite.hEvent == NULL)
	{
		DWORD dwError = GetLastError();
		if(m_OverlappedRead.hEvent != NULL) CloseHandle(m_OverlappedRead.hEvent);
		if(m_OverlappedWrite.hEvent != NULL) CloseHandle(m_OverlappedWrite.hEvent);
		CloseHandle(m_hIDComDev);
		return false;
	}

	m_bOpened = true;

	Purge();

	return m_bOpened;
}

bool CSerial::Close(void)
{
	if(!m_bOpened || m_hIDComDev == NULL)
		return true;

	if(m_OverlappedRead.hEvent != NULL) 
		CloseHandle(m_OverlappedRead.hEvent);

	if(m_OverlappedWrite.hEvent != NULL)
		CloseHandle(m_OverlappedWrite.hEvent);

	CloseHandle(m_hIDComDev);
	m_bOpened = false;
	m_hIDComDev = NULL;

	return true;
}

bool CSerial::WriteCommByte(unsigned char ucByte)
{
	BOOL bWriteStat;
	DWORD dwBytesWritten;

	bWriteStat = WriteFile(m_hIDComDev, (LPSTR) &ucByte, 1, &dwBytesWritten, &m_OverlappedWrite);

	if(!bWriteStat)
	{
		int nErr = GetLastError();
		if(nErr == ERROR_IO_PENDING)
		{
			if(WaitForSingleObject(m_OverlappedWrite.hEvent, 1000))
				dwBytesWritten = 0;
			else
			{
				GetOverlappedResult(m_hIDComDev, &m_OverlappedWrite, &dwBytesWritten, false);
				m_OverlappedWrite.Offset += dwBytesWritten;
			}
		}
		else // if(nErr == ERROR_NOACCESS || nErr == ERROR_ACCESS_DENIED)
		{
			Close();
			return false;
		}
	}

	return true;
}

int CSerial::SendData(const char *buffer, int size)
{
	if(!m_bOpened || m_hIDComDev == NULL) 
		return 0;

	DWORD dwBytesWritten = 0;
	int i;
	for(i=0; i<size; i++)
	{
		if(!WriteCommByte(buffer[i]))
			break;
		dwBytesWritten++;
	}

	return (int)dwBytesWritten;

}

int CSerial::ReadDataWaiting(void)
{
	if(!m_bOpened || m_hIDComDev == NULL) 
		return 0;

	DWORD dwErrorFlags;
	COMSTAT ComStat;

	ClearCommError(m_hIDComDev, &dwErrorFlags, &ComStat);

	return (int)ComStat.cbInQue;
}

int CSerial::ReadData(void *buffer, int limit, int nTimeout)
{
	if(!m_bOpened || m_hIDComDev == NULL) 
		return 0;

	BOOL bReadStatus;
	DWORD dwBytesRead, dwErrorFlags;
	COMSTAT ComStat;

	ClearCommError(m_hIDComDev, &dwErrorFlags, &ComStat);
	if(!ComStat.cbInQue) 
		return 0;

	dwBytesRead = (DWORD) ComStat.cbInQue;

	if(limit < (int) dwBytesRead) 
		dwBytesRead = (DWORD) limit;

	bReadStatus = ReadFile(m_hIDComDev, buffer, dwBytesRead, &dwBytesRead, &m_OverlappedRead);
	
	if(!bReadStatus)
	{
		int nErr = GetLastError();
		if(nErr == ERROR_IO_PENDING)
		{
			WaitForSingleObject(m_OverlappedRead.hEvent, nTimeout);
			return (int)dwBytesRead;
		}
		else if(nErr == ERROR_NOACCESS)
			Close();

		return 0;
	}

	return (int)dwBytesRead;

}

CStringA CSerial::ReadString(int nTimeout)
{
	CStringA str;
	char szBuf[2];
	DWORD dwEndTime = GetTickCount() + nTimeout;
	bool bGotOn = false;

	while(1)
	{
		bGotOn = false;
		if(ReadData(szBuf, 1, 50) == 1)
		{
			if(*szBuf == '\r')
				break;

			if(*szBuf == '\n')
				continue;

			str += *szBuf;
		}
		else if(nTimeout <= 0)
			break;

		if(nTimeout > 0 && GetTickCount() >= dwEndTime)
			break;
		else if(!bGotOn && nTimeout > 0 && (dwEndTime - GetTickCount()) > 16)	// It we have at least 16ms of time left, then Sleep(1), although it'll typically really be ~16ms
			Sleep(1);
	}

	return str;
}

void CSerial::Purge()
{
	if(!m_bOpened || m_hIDComDev == NULL) 
		return;

	PurgeComm(m_hIDComDev, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);//PURGE_RXABORT | PURGE_RXCLEAR);
}
